//************************************************************
// Funktionen fuer den Temperatursensor LM75A
//
//Controller:	ATmega8 or ATmega328
//Author:	Rolf Pfister
//		www.rolfp.ch
//Date:		Juni 2011

//11.6.2011   Erstellung
//1.10.2011   Erweiterung fuer Sleep-Modus
//14.10.2011  Anderer Fehlercode in Funktion lm75a_readt(), so
//            dass, wenn nicht auf Fehler geprueft wird, eine
//            ungueltige Temperatur rauskommt.

//************************************************************
#define F_CPU 3686411
#include "lm75a.h"
#include "i2c.h"
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
//#include "delay.cc"

static char aktadr=0;
//static char nr2=0;  //Nummer des Fuehlers mal 2

//void setfuehlernummer(char nr)
//{
// nr2=2*nr;
//}

unsigned char lm75a_i2c_startw(char adr)
{
 unsigned char error;
 error = i2c_start();                   if(error) {return error;}
 error = i2c_sendAddress(LM75A_W/*+nr2*/);  if(error) {return error;}
 return  i2c_sendData(aktadr=adr);
}

unsigned char lm75a_i2c_startr()
{
 unsigned char error;
 error = i2c_start();                   
 if(error) {return error;}

 return  i2c_sendAddress(LM75A_R/*+nr2*/);
}

//***********************************************************************
//Function to write LM75A registers
//***********************************************************************
unsigned char lm75a_write(char *data,char adr,unsigned char imax)
{
 unsigned char error, i;
 error = lm75a_i2c_startw(adr);     if(error) {i2c_stop(); return error;}
 for(i=0;i<imax;i++)
   {
    error = i2c_sendData(data[i]);  if(error) {i2c_stop(); return error;}
   }
 i2c_stop();
 return LM75A_NO_ERROR;
}

//***********************************************************************
//Function to read Temperatur from LM75A
//***********************************************************************
int lm75a_readt(char adr)     //Temperatur in 8-tel Grad
{
 //unsigned char error; 
 char data[2]; 
 int temperatur;
 
 //if(aktadr!=adr)
  //{
   lm75a_i2c_startw(adr); 
   i2c_stop(); 
   //if(error) return LM75A_ERROR;
   _delay_ms(1);
  //}
 //error = lm75a_i2c_startr();   
 //if(error) {i2c_stop(); 
 //return LM75A_ERROR;}
 //data[1] = i2c_receiveData_ACK();
 //data[0] = i2c_receiveData_NACK();

 i2c_stop();
 temperatur = data[1];
 temperatur <<= 3;
 temperatur += (data[0]>>5)&0x07;
 if(temperatur&0x400) temperatur |= 0xF800;
 return temperatur;
}

int lm75a_readtemp()     //aktuelle Temperatur in 8-tel Grad
{
 return lm75a_readt(LM75A_ADR_Temp);
}
int lm75a_readtos()      //Oberer Sollwert in 8-tel Grad
{
 return lm75a_readt(LM75A_ADR_Tos);
}
int lm75a_readthyst()    //Unterer Sollwert in 8-tel Grad
{
 return lm75a_readt(LM75A_ADR_Thyst);
}


//***********************************************************************
//Function to write Tos and Thyst to LM75A
//***********************************************************************
unsigned char lm75a_sollwrite(int tos,int thyst)  //Sollwerte in 8-tel Grad
{
 char data[2];
 unsigned char error;
 data[0] = tos>>3;
 data[1] = (tos<<5)&0xE0;
 error = lm75a_write(data,LM75A_ADR_Tos,2);   if(error) return ERROR_CODE;
 _delay_ms(1);
 data[0] = thyst>>3;
 data[1] = (thyst<<5)&0xE0;
 return lm75a_write(data,LM75A_ADR_Thyst,2);
}

//***********************************************************************
//Functions to read and write Conf register of LM75A
//***********************************************************************
unsigned char lm75a_readconf()           //Configurations-Register lesen
{
 unsigned char error; char conf;
 //if(aktadr!=LM75A_ADR_Conf)
  {
   error=lm75a_i2c_startw(LM75A_ADR_Conf); i2c_stop(); if(error) return ERROR_CODE;
   _delay_ms(1);
  }
 error = lm75a_i2c_startr();   if(error) {i2c_stop(); return ERROR_CODE;}
 conf = i2c_receiveData_NACK();
 i2c_stop();
 return conf;
}

unsigned char lm75a_writeconf(char conf) //Configurations-Register schreiben
{
 char data[2];
 data[0]=conf;
 return lm75a_write(data,LM75A_ADR_Conf,1);
}


void lm75a_sleep()
{
 lm75a_writeconf(0x01);
}

void lm75a_nosleep()
{
 lm75a_writeconf(0x00);
}
